@@ -28,7 +28,7 @@ __all__ = [
     "visualize",
     "calculus",
     "multi_optimize",
-    "one_optimize",
+    "one_optimize"
 ]
 
-__version__ = '1.0 dev 2022 ))) Hi'
+__version__ = '1.0 dev 2048 ))) Hi'
