@@ -1,13 +1,13 @@
 # One dimensional optimization algorithms for function optimization
 from __future__ import annotations
 
+import sys
 from typing import Tuple, Callable
 
-try:  # compatibility with Python37
+if sys.version_info >= (3, 8):
     from typing import Literal
-except ImportError:
+else:
     from typing_extensions import Literal
-
 import torch
 
 from .support import HistoryGSS, HistorySPI, HistoryBrent, update_history_brent
@@ -43,12 +43,6 @@ def golden_section_search(function: Callable[[float | torch.Tensor], float],
     .. note::
         If optimization fails golden_section_search will return the last point
 
-    Code example:
-
-    .. code-block:: python3
-
-        >>> def func(x): return 2.71828 ** (3 * x) + 5 * 2.71828 ** (-2 * x)
-        >>> point, data = golden_section_search(func, (-10, 10), type_optimization='min', keep_history=True)
 
     .. rubric:: References
 
@@ -67,6 +61,12 @@ def golden_section_search(function: Callable[[float | torch.Tensor], float],
     :param keep_history: flag of return history
     :return: tuple with point and history.
 
+    .. rubric:: Examples
+
+    .. code-block:: python3
+
+        >>> def func(x): return 2.71828 ** (3 * x) + 5 * 2.71828 ** (-2 * x)
+        >>> point, data = golden_section_search(func, (-10, 10), type_optimization='min', keep_history=True)
     """
     phi: float = (1 + 5 ** 0.5) / 2
 
@@ -86,8 +86,8 @@ def golden_section_search(function: Callable[[float | torch.Tensor], float],
         history: HistoryGSS = {'iteration': [], 'middle_point': [], 'f_value': [], 'left_point': [], 'right_point': []}
 
     if verbose:
-        print(f'Iteration: {0} \\t|\\t point = {(a + b) / 2 :0.3f} '
-              f'\\t|\\t f(point) = {function((a + b) / 2): 0.3f}')
+        print(f'Iteration: {0} \t|\t point = {(a + b) / 2 :0.3f} '
+              f'\t|\t f(point) = {function((a + b) / 2): 0.3f}')
 
     try:
         for i in range(1, max_iter):
@@ -107,8 +107,8 @@ def golden_section_search(function: Callable[[float | torch.Tensor], float],
 
             middle_point: float = (a + b) / 2
             if verbose:
-                print(f'Iteration: {i} \\t|\\t point = {middle_point :0.3f} '
-                      f'\\t|\\t f(point) = {function(middle_point): 0.3f}')
+                print(f'Iteration: {i} \t|\t point = {middle_point :0.3f} '
+                      f'\t|\t f(point) = {function(middle_point): 0.3f}')
 
             if keep_history:
                 history['iteration'].append(i)
@@ -168,7 +168,25 @@ def successive_parabolic_interpolation(function: Callable[[float | torch.Tensor]
     :math:`\\rule{125mm}{0.7pt} \\\\`
 
 
-    Example:
+    ..  [3] Press, William H., Teukolsky, S. A., Vetterling, W. T., & Flannery, B. P. (2007).
+        Numerical Recipes with Source Code CD-ROM 3rd Edition: The Art of Scientific Computing (3rd ed.).
+        Cambridge University Press. p.496-499
+
+    :math:`\\rule{125mm}{0.2pt} \\\\`
+
+    :param function: callable that depends on the first positional argument. Other arguments are passed through kwargs
+    :param bounds: tuple with two numbers. This is left and right bound optimization. [a, b]
+    :param epsilon: optimization accuracy
+    :param type_optimization: 'min' / 'max' - type of required value
+    :param max_iter: maximum number of iterations
+    :param verbose: flag of printing iteration logs
+    :param keep_history: flag of return history
+    :return: tuple with point and history.
+
+    .. rubric:: Examples
+
+    .. code-block:: python3
+
         >>> def func1(x): return x ** 3 - x ** 2 - x
         >>> successive_parabolic_interpolation(func1, (0, 1.5), verbose=True)
         Iteration: 0	|	x2 = 0.750	|	f(x2) = -0.891
@@ -185,22 +203,6 @@ def successive_parabolic_interpolation(function: Callable[[float | torch.Tensor]
         Iteration: 2	|	x2 = 0.961	|	f(x2) =  0.997
         Iteration: 3	|	x2 = 1.017	|	f(x2) =  0.999
         ...
-
-    ..  [3] Press, William H., Teukolsky, S. A., Vetterling, W. T., & Flannery, B. P. (2007).
-        Numerical Recipes with Source Code CD-ROM 3rd Edition: The Art of Scientific Computing (3rd ed.).
-        Cambridge University Press. p.496-499
-
-    :math:`\\rule{125mm}{0.2pt} \\\\`
-
-    :param function: callable that depends on the first positional argument. Other arguments are passed through kwargs
-    :param bounds: tuple with two numbers. This is left and right bound optimization. [a, b]
-    :param epsilon: optimization accuracy
-    :param type_optimization: 'min' / 'max' - type of required value
-    :param max_iter: maximum number of iterations
-    :param verbose: flag of printing iteration logs
-    :param keep_history: flag of return history
-    :return: tuple with point and history.
-
     """
     type_optimization = type_optimization.lower().strip()
     assert type_optimization in ['min', 'max'], 'Invalid type optimization. Enter "min" or "max"'
@@ -212,11 +214,12 @@ def successive_parabolic_interpolation(function: Callable[[float | torch.Tensor]
 
     history: HistorySPI = {'iteration': [], 'f_value': [], 'x0': [], 'x1': [], 'x2': []}
     x0, x1 = bounds[0], bounds[1]
-    x2 = (x0 + x1) / 2
+    x2 = (x0 + x1) / 3 * 2
     f0 = type_opt_const * function(x0)
     f1 = type_opt_const * function(x1)
     f2 = type_opt_const * function(x2)
     f_x: dict = {x0: f0, x1: f1, x2: f2}
+    count_upd_point: int = 0  # if the numerator or denominator is zero, we will choose a new point. max of 100 times
     x2, x1, x0 = sorted([x0, x1, x2], key=lambda x: f_x[x])
 
     if keep_history:
@@ -235,12 +238,15 @@ def successive_parabolic_interpolation(function: Callable[[float | torch.Tensor]
             p = (x1 - x2) ** 2 * (f2 - f0) + (x0 - x2) ** 2 * (f1 - f2)
             q = 2 * ((x1 - x2) * (f2 - f0) + (x0 - x2) * (f1 - f2))
 
-            if p != 0:
-                print('Searching finished. Select an another initial state. Numerator is zero. code 2')
-                return x2, history
-            if q != 0:
-                print('Searching finished. Select an another initial state. Denominator is zero. code 2')
-                return x2, history
+            if p == 0 or q == 0:
+                if count_upd_point >= 100:
+                    print('Select an another initial state. Numerator or Denominator is zero. code 2')
+                    return x2, history
+                else:
+                    count_upd_point += 1
+                    x2 = float(torch.rand(1) * (abs(x0 - x1)) * min(x0, x1))
+                    f_x[x2] = function(x2)
+                    continue
 
             x_new = x2 + p / q
 
@@ -316,7 +322,7 @@ def brent(function: Callable[[float | torch.Tensor], float],
     :math:`\\qquad \\text{if } \\displaystyle remainder < previous \\ remainder \\ \\& \\
     x_{least} + remainder \\in (a, b):\\\\`
 
-    :math:`\\qquad \\qquad  \\text{use  ``paraboloic" } \\displaystyle remainder\\\\`
+    :math:`\\qquad \\qquad  \\text{use  ``parabolic" } \\displaystyle remainder\\\\`
 
     :math:`\\qquad \\text{else:}\\\\`
     :math:`\\qquad \\qquad \\text{make  ``golden"  } \\displaystyle remainder\\\\`
@@ -343,6 +349,28 @@ def brent(function: Callable[[float | torch.Tensor], float],
     :param keep_history: flag of return history
     :return: tuple with point and history.
 
+
+    .. rubric:: Examples
+
+    .. code-block:: python3
+
+        >>> brent(lambda x: x ** 3 - x ** 2 - x, (0,2), verbose=True)[0]
+        iteration 0	x = 0.763932,	f(x) = -0.901699	type : initial
+        iteration 1	x = 0.763932,	f(x) = -0.901699	type : golden
+        iteration 2	x = 0.763932,	f(x) = -0.901699	type : golden
+        iteration 3	x = 0.944272,	f(x) = -0.993962	type : golden
+        iteration 4	x = 0.944272,	f(x) = -0.993962	type : golden
+        iteration 5	x = 0.999120,	f(x) = -0.999998	type : parabolic
+        iteration 6	x = 0.999223,	f(x) = -0.999999	type : parabolic
+        iteration 7	x = 0.999223,	f(x) = -0.999999	type : golden
+        iteration 8	x = 0.999992,	f(x) = -1.000000	type : parabolic
+        iteration 9	x = 1.000002,	f(x) = -1.000000	type : parabolic
+        iteration 10	x = 1.000002,	f(x) = -1.000000	type : golden
+        iteration 11	x = 1.000002,	f(x) = -1.000000	type : parabolic
+        Searching finished. Successfully. code 0
+        1.0000016327177492
+
+
     """
 
     type_optimization = type_optimization.lower().strip()
@@ -486,17 +514,18 @@ def brent(function: Callable[[float | torch.Tensor], float],
         if keep_history:
             history = update_history_brent(
                 history,
-                [i,
-                 type_opt_const * f_least,
-                 type_opt_const * f_middle,
-                 type_opt_const * f_largest,
-                 x_least,
-                 x_middle,
-                 x_largest,
-                 a,
-                 b,
-                 name_step
-                 ]
+                [
+                    i,
+                    type_opt_const * f_least,
+                    type_opt_const * f_middle,
+                    type_opt_const * f_largest,
+                    x_least,
+                    x_middle,
+                    x_largest,
+                    a,
+                    b,
+                    name_step
+                ]
             )
 
         if verbose:
@@ -505,5 +534,3 @@ def brent(function: Callable[[float | torch.Tensor], float],
     else:
         print('Searching finished. Max iterations have been reached. code 1')
         return x_least, history
-
-# enf of One dimensional part
